@@ -1,12 +1,29 @@
 //! # APIs bridging OSTree and container images
 //!
-//! This crate contains APIs to bidirectionally map
-//! between OSTree repositories and container images.
-
-//#![deny(missing_docs)]
-// Good defaults
-#![forbid(unused_must_use)]
-#![deny(unsafe_code)]
+//! This module contains APIs to bidirectionally map between a single OSTree commit and a container image wrapping it.
+//! Because container images are just layers of tarballs, this builds on the [`crate::tar`] module.
+//!
+//! To emphasize this, the current high level model is that this is a one-to-one mapping - an ostree commit
+//! can be exported (wrapped) into a container image, which will have exactly one layer.  Upon import
+//! back into an ostree repository, all container metadata except for its digested checksum will be discarded.
+//!
+//! ## Signatures
+//!
+//! OSTree supports GPG and ed25519 signatures natively, and it's expected by default that
+//! when booting from a fetched container image, one verifies ostree-level signatures.
+//! For ostree, a signing configuration is specified via an ostree remote.  In order to
+//! pair this configuration together, this library defines a "URL-like" string schema:
+//!
+//! `ostree-remote-registry:<remotename>:<containerimage>`
+//!
+//! A concrete instantiation might be e.g.: `ostree-remote-registry:fedora:quay.io/coreos/fedora-coreos:stable`
+//!
+//! To parse and generate these strings, see [`OstreeImageReference`].
+//!
+//! ## Layering
+//!
+//! A key feature of container images is support for layering.  At the moment, support
+//! for this is [planned but not implemented](https://github.com/ostreedev/ostree-rs-ext/issues/12).
 
 use anyhow::anyhow;
 use std::borrow::Cow;
@@ -20,12 +37,6 @@ pub const OSTREE_COMMIT_LABEL: &str = "ostree.commit";
 /// to a string to output to a terminal or logs.
 type Result<T> = anyhow::Result<T>;
 
-/// Information about the image manifest.
-pub struct OstreeContainerManifestInfo {
-    /// The manifest digest (`sha256:<value>`)
-    pub manifest_digest: String,
-}
-
 /// A backend/transport for OCI/Docker images.
 #[derive(Copy, Clone, Debug, PartialEq, Eq)]
 pub enum Transport {
@@ -61,60 +72,16 @@ pub enum SignatureSource {
     ContainerPolicyAllowInsecure,
 }
 
-/// Combination of an ostree remote (for signature verification) and an image reference.
+/// Combination of a signature verification mechanism, and a standard container image reference.
 ///
-/// For example, myremote:docker://quay.io/somerepo/someimage.latest
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub struct OstreeImageReference {
-    /// The ostree remote name.
-    /// This will be used for signature verification.
+    /// The signature verification mechanism.
     pub sigverify: SignatureSource,
     /// The container image reference.
     pub imgref: ImageReference,
 }
 
-impl ImageReference {
-    /// Create a new `ImageReference` that refers to a specific digest.
-    ///
-    /// ```rust
-    /// use std::convert::TryInto;
-    /// let r: ostree_ext::container::ImageReference = "docker://quay.io/exampleos/exampleos:latest".try_into().unwrap();
-    /// let n = r.with_digest("sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-    /// assert_eq!(n.name, "quay.io/exampleos/exampleos@sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-    /// ```
-    pub fn with_digest(&self, digest: &str) -> Self {
-        let name = self.name.as_str();
-        let name = if let Some(idx) = name.rfind('@') {
-            name.split_at(idx).0
-        } else if let Some(idx) = name.rfind(':') {
-            name.split_at(idx).0
-        } else {
-            name
-        };
-        Self {
-            transport: self.transport,
-            name: format!("{}@{}", name, digest),
-        }
-    }
-}
-
-impl OstreeImageReference {
-    /// Create a new `OstreeImageReference` that refers to a specific digest.
-    ///
-    /// ```rust
-    /// use std::convert::TryInto;
-    /// let r: ostree_ext::container::OstreeImageReference = "ostree-remote-image:myremote:docker://quay.io/exampleos/exampleos:latest".try_into().unwrap();
-    /// let n = r.with_digest("sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-    /// assert_eq!(n.imgref.name, "quay.io/exampleos/exampleos@sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-    /// ```
-    pub fn with_digest(&self, digest: &str) -> Self {
-        Self {
-            sigverify: self.sigverify.clone(),
-            imgref: self.imgref.with_digest(digest),
-        }
-    }
-}
-
 impl TryFrom<&str> for Transport {
     type Error = anyhow::Error;
 
@@ -185,6 +152,11 @@ impl TryFrom<&str> for OstreeImageReference {
                 SignatureSource::ContainerPolicyAllowInsecure,
                 Cow::Borrowed(second),
             ),
+            // Shorthand for ostree-unverified-image:registry:
+            "ostree-unverified-registry" => (
+                SignatureSource::ContainerPolicyAllowInsecure,
+                Cow::Owned(format!("registry:{}", second)),
+            ),
             // This is a shorthand for ostree-remote-image with registry:
             "ostree-remote-registry" => {
                 let mut subparts = second.splitn(2, ':');
@@ -251,12 +223,14 @@ impl std::fmt::Display for OstreeImageReference {
     }
 }
 
-mod export;
-pub use export::*;
-mod import;
-pub use import::*;
-mod oci;
+pub mod deploy;
+mod encapsulate;
+pub use encapsulate::*;
+mod unencapsulate;
+pub use unencapsulate::*;
+mod ociwriter;
 mod skopeo;
+pub mod store;
 
 #[cfg(test)]
 mod tests {
@@ -275,18 +249,6 @@ mod tests {
         assert_eq!(ir.name, "quay.io/exampleos/blah");
         assert_eq!(ir.to_string(), "docker://quay.io/exampleos/blah");
 
-        let digested = ir
-            .with_digest("sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-        assert_eq!(digested.name, "quay.io/exampleos/blah@sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-        assert_eq!(digested.with_digest("sha256:52f562806109f5746be31ccf21f5569fd2ce8c32deb0d14987b440ed39e34e20").name, "quay.io/exampleos/blah@sha256:52f562806109f5746be31ccf21f5569fd2ce8c32deb0d14987b440ed39e34e20");
-
-        let with_tag: ImageReference = "registry:quay.io/exampleos/blah:sometag"
-            .try_into()
-            .unwrap();
-        let digested = with_tag
-            .with_digest("sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-        assert_eq!(digested.name, "quay.io/exampleos/blah@sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-
         for &v in VALID_IRS {
             ImageReference::try_from(v).unwrap();
         }
@@ -325,11 +287,6 @@ mod tests {
         // test our Eq implementation
         assert_eq!(&ir, &OstreeImageReference::try_from(ir_registry).unwrap());
 
-        let digested = ir
-            .with_digest("sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-        assert_eq!(digested.imgref.name, "quay.io/exampleos/blah@sha256:41af286dc0b172ed2f1ca934fd2278de4a1192302ffa07087cea2682e7d372e3");
-        assert_eq!(digested.with_digest("sha256:52f562806109f5746be31ccf21f5569fd2ce8c32deb0d14987b440ed39e34e20").imgref.name, "quay.io/exampleos/blah@sha256:52f562806109f5746be31ccf21f5569fd2ce8c32deb0d14987b440ed39e34e20");
-
         let ir_s = "ostree-image-signed:docker://quay.io/exampleos/blah";
         let ir: OstreeImageReference = ir_s.try_into().unwrap();
         assert_eq!(ir.sigverify, SignatureSource::ContainerPolicy);
@@ -349,5 +306,9 @@ mod tests {
             ir.to_string(),
             "ostree-unverified-image:docker://quay.io/exampleos/blah"
         );
+        let ir_shorthand =
+            OstreeImageReference::try_from("ostree-unverified-registry:quay.io/exampleos/blah")
+                .unwrap();
+        assert_eq!(&ir_shorthand, &ir);
     }
 }
