@@ -13,6 +13,7 @@
 // Re-export our dependencies.  See https://gtk-rs.org/blog/2021/06/22/new-release.html
 // "Dependencies are re-exported".  Users will need e.g. `gio::File`, so this avoids
 // them needing to update matching versions.
+pub use oci_spec;
 pub use ostree;
 pub use ostree::gio;
 pub use ostree::gio::glib;
@@ -26,7 +27,13 @@ pub mod cli;
 pub mod container;
 pub mod diff;
 pub mod ima;
+pub mod keyfileext;
+pub mod refescape;
 pub mod tar;
+pub mod tokio_util;
+
+mod cmdext;
+pub(crate) mod objgv;
 /// Prelude, intended for glob import.
 pub mod prelude {
     #[doc(hidden)]
